@@ -9,6 +9,7 @@ import io
 import sys
 import platform
 import warnings
+from math import cos, sin, tan, atan2, sqrt, radians, degrees, pi, log, fmod
 
 # Some platforms (CYGWIN and others) many not have the wx library
 # use imp to see if wx is on the path
@@ -27,38 +28,46 @@ else:
 
 radius_of_earth = 6378100.0 # in meters
 
+def wrap_360(angle):
+    '''wrap an angle to 0..360 degrees'''
+    return math.fmod(angle, 360.0)
+
+def wrap_180(angle):
+    '''wrap an angle to -180..180 degrees'''
+    if angle >= -180 and angle < 180:
+        return angle
+    a = math.fmod(angle+180, 360.0)-180.0
+    return a
+
 def gps_distance(lat1, lon1, lat2, lon2):
-    '''return distance between two points in meters,
+    '''distance between two points in meters along rhumb line
     coordinates are in degrees
-    thanks to http://www.movable-type.co.uk/scripts/latlong.html'''
-    lat1 = math.radians(lat1)
-    lat2 = math.radians(lat2)
-    lon1 = math.radians(lon1)
-    lon2 = math.radians(lon2)
-    dLat = lat2 - lat1
-    dLon = lon2 - lon1
-
-    a = math.sin(0.5*dLat)**2 + math.sin(0.5*dLon)**2 * math.cos(lat1) * math.cos(lat2)
-    c = 2.0 * math.atan2(math.sqrt(a), math.sqrt(1.0-a))
-    return radius_of_earth * c
+    thanks to http://www.movable-type.co.uk/scripts/latlong.html
+    and http://www.edwilliams.org/avform147.htm#Rhumb
+    '''
+    lat1 = radians(lat1)
+    lat2 = radians(lat2)
+    lon1 = radians(lon1)
+    lon2 = radians(lon2)
 
+    if abs(lat2-lat1) < 1.0e-15:
+        q = cos(lat1)
+    else:
+        q = (lat2-lat1)/log(tan(lat2/2+pi/4)/tan(lat1/2+pi/4))
+    d = sqrt((lat2-lat1)**2 + q**2 * (lon2-lon1)**2)
+    return d * radius_of_earth
 
 def gps_bearing(lat1, lon1, lat2, lon2):
-    '''return bearing between two points in degrees, in range 0-360
+    '''return rhumb bearing between two points in degrees, in range 0-360
     thanks to http://www.movable-type.co.uk/scripts/latlong.html'''
-    lat1 = math.radians(lat1)
-    lat2 = math.radians(lat2)
-    lon1 = math.radians(lon1)
-    lon2 = math.radians(lon2)
-    dLat = lat2 - lat1
-    dLon = lon2 - lon1
-    y = math.sin(dLon) * math.cos(lat2)
-    x = math.cos(lat1)*math.sin(lat2) - math.sin(lat1)*math.cos(lat2)*math.cos(dLon)
-    bearing = math.degrees(math.atan2(y, x))
-    if bearing < 0:
-        bearing += 360.0
-    return bearing
-
+    lat1 = radians(lat1)
+    lat2 = radians(lat2)
+    lon1 = radians(lon1)
+    lon2 = radians(lon2)
+    tc = -fmod(atan2(lon1-lon2,log(tan(lat2/2+pi/4)/tan(lat1/2+pi/4))),2*pi)
+    if tc < 0:
+        tc += 2*pi
+    return degrees(tc)
 
 def wrap_valid_longitude(lon):
     ''' wrap a longitude value around to always have a value in the range
@@ -66,26 +75,46 @@ def wrap_valid_longitude(lon):
     '''
     return (((lon + 180.0) % 360.0) - 180.0)
 
+def constrain(v, minv, maxv):
+    if v < minv:
+        v = minv
+    if v > maxv:
+        v = maxv
+    return v
+
+def constrain_latlon(latlon):
+    epsilon = 1.0e-3
+    return (constrain(latlon[0],-90+epsilon, 90-epsilon), wrap_180(latlon[1]))
+
 def gps_newpos(lat, lon, bearing, distance):
     '''extrapolate latitude/longitude given a heading and distance
-    thanks to http://www.movable-type.co.uk/scripts/latlong.html
+    along rhumb line thanks to http://www.movable-type.co.uk/scripts/latlong.html
     '''
-    lat1 = math.radians(lat)
-    lon1 = math.radians(lon)
-    brng = math.radians(bearing)
-    dr = distance/radius_of_earth
+    lat1 = constrain(radians(lat), -pi/2+1.0e-15, pi/2-1.0e-15)
+    lon1 = radians(lon)
+    tc = radians(-bearing)
+    d = distance/radius_of_earth
 
-    lat2 = math.asin(math.sin(lat1)*math.cos(dr) +
-                     math.cos(lat1)*math.sin(dr)*math.cos(brng))
-    lon2 = lon1 + math.atan2(math.sin(brng)*math.sin(dr)*math.cos(lat1),
-                             math.cos(dr)-math.sin(lat1)*math.sin(lat2))
-    return (math.degrees(lat2), wrap_valid_longitude(math.degrees(lon2)))
+    lat = lat1 + d * cos(tc)
+    lat = constrain(lat, -pi/2 + 1.0e-15, pi/2 - 1.0e-15)
+    if abs(lat-lat1) < 1.0e-15:
+        q = cos(lat1)
+    else:
+        try:
+            dphi = log(tan(lat/2+pi/4)/tan(lat1/2+pi/4))
+        except Exception:
+            print(degrees(lat),degrees(lat1))
+            raise
+        q = (lat-lat1)/dphi
+    dlon = -d*sin(tc)/q
+    lon = fmod(lon1+dlon+pi,2*pi)-pi
+    return (degrees(lat), degrees(lon))
 
 def gps_offset(lat, lon, east, north):
     '''return new lat/lon after moving east/north
     by the given number of meters'''
-    bearing = math.degrees(math.atan2(east, north))
-    distance = math.sqrt(east**2 + north**2)
+    bearing = degrees(atan2(east, north))
+    distance = sqrt(east**2 + north**2)
     return gps_newpos(lat, lon, bearing, distance)
 
 
@@ -134,16 +163,16 @@ def polygon_bounds(points):
     return (minx, miny, maxx-minx, maxy-miny)
 
 def bounds_overlap(bound1, bound2):
-    '''return true if two bounding boxes overlap'''
+    '''return true if two bounding boxes overlap. coordinates are lat/lon degrees. x is lat, y is lon'''
     (x1,y1,w1,h1) = bound1
     (x2,y2,w2,h2) = bound2
     if x1+w1 < x2:
         return False
     if x2+w2 < x1:
         return False
-    if y1+h1 < y2:
+    if wrap_180((y1+h1)-y2) < 0:
         return False
-    if y2+h2 < y1:
+    if wrap_180((y2+h2)-y1) < 0:
         return False
     return True
 
@@ -191,6 +220,7 @@ class UTMGrid:
         '''return (lat,lon) for the grid coordinates'''
         from MAVProxy.modules.lib.ANUGA import lat_long_UTM_conversion
         (lat, lon) = lat_long_UTM_conversion.UTMtoLL(self.northing, self.easting, self.zone, isSouthernHemisphere=(self.hemisphere=='S'))
+        lon = wrap_180(lon)
         return (lat, lon)
 
 
@@ -352,69 +382,83 @@ def decode_devid(devid, pname):
         }
 
     compass_types = {
-        0x01 : "DEVTYPE_HMC5883_OLD",
-        0x07 : "DEVTYPE_HMC5883",
-        0x02 : "DEVTYPE_LSM303D",
-        0x04 : "DEVTYPE_AK8963 ",
-        0x05 : "DEVTYPE_BMM150 ",
-        0x06 : "DEVTYPE_LSM9DS1",
-        0x08 : "DEVTYPE_LIS3MDL",
-        0x09 : "DEVTYPE_AK09916",
-        0x0A : "DEVTYPE_IST8310",
-        0x0B : "DEVTYPE_ICM20948",
-        0x0C : "DEVTYPE_MMC3416",
-        0x0D : "DEVTYPE_QMC5883L",
-        0x0E : "DEVTYPE_MAG3110",
-        0x0F : "DEVTYPE_SITL",
-        0x10 : "DEVTYPE_IST8308",
-        0x11 : "DEVTYPE_RM3100",
-        0x12 : "DEVTYPE_RM3100_2",
+        0x01 : "HMC5883_OLD",
+        0x07 : "HMC5883",
+        0x02 : "LSM303D",
+        0x04 : "AK8963 ",
+        0x05 : "BMM150 ",
+        0x06 : "LSM9DS1",
+        0x08 : "LIS3MDL",
+        0x09 : "AK09916",
+        0x0A : "IST8310",
+        0x0B : "ICM20948",
+        0x0C : "MMC3416",
+        0x0D : "QMC5883L",
+        0x0E : "MAG3110",
+        0x0F : "SITL",
+        0x10 : "IST8308",
+        0x11 : "RM3100",
+        0x12 : "RM3100_2",
         }
 
     imu_types = {
-        0x09 : "DEVTYPE_BMI160",
-        0x10 : "DEVTYPE_L3G4200D",
-        0x11 : "DEVTYPE_ACC_LSM303D",
-        0x12 : "DEVTYPE_ACC_BMA180",
-        0x13 : "DEVTYPE_ACC_MPU6000",
-        0x16 : "DEVTYPE_ACC_MPU9250",
-        0x17 : "DEVTYPE_ACC_IIS328DQ",
-        0x18 : "DEVTYPE_ACC_LSM9DS1",
-        0x21 : "DEVTYPE_GYR_MPU6000",
-        0x22 : "DEVTYPE_GYR_L3GD20",
-        0x24 : "DEVTYPE_GYR_MPU9250",
-        0x25 : "DEVTYPE_GYR_I3G4250D",
-        0x26 : "DEVTYPE_GYR_LSM9DS1",
-        0x27 : "DEVTYPE_INS_ICM20789",
-        0x28 : "DEVTYPE_INS_ICM20689",
-        0x29 : "DEVTYPE_INS_BMI055",
-        0x2A : "DEVTYPE_SITL",
-        0x2B : "DEVTYPE_INS_BMI088",
-        0x2C : "DEVTYPE_INS_ICM20948",
-        0x2D : "DEVTYPE_INS_ICM20648",
-        0x2E : "DEVTYPE_INS_ICM20649",
-        0x2F : "DEVTYPE_INS_ICM20602",
-        0x30 : "DEVTYPE_INS_ICM20601",
-        0x31 : "DEVTYPE_INS_ADIS1647x",
-        0x32 : "DEVTYPE_INS_SERIAL",
-        0x33 : "DEVTYPE_INS_ICM40609",
-        0x34 : "DEVTYPE_INS_ICM42688",
+        0x09 : "BMI160",
+        0x10 : "L3G4200D",
+        0x11 : "ACC_LSM303D",
+        0x12 : "ACC_BMA180",
+        0x13 : "ACC_MPU6000",
+        0x16 : "ACC_MPU9250",
+        0x17 : "ACC_IIS328DQ",
+        0x18 : "ACC_LSM9DS1",
+        0x21 : "GYR_MPU6000",
+        0x22 : "GYR_L3GD20",
+        0x24 : "GYR_MPU9250",
+        0x25 : "GYR_I3G4250D",
+        0x26 : "GYR_LSM9DS1",
+        0x27 : "INS_ICM20789",
+        0x28 : "INS_ICM20689",
+        0x29 : "INS_BMI055",
+        0x2A : "SITL",
+        0x2B : "INS_BMI088",
+        0x2C : "INS_ICM20948",
+        0x2D : "INS_ICM20648",
+        0x2E : "INS_ICM20649",
+        0x2F : "INS_ICM20602",
+        0x30 : "INS_ICM20601",
+        0x31 : "INS_ADIS1647x",
+        0x32 : "INS_SERIAL",
+        0x33 : "INS_ICM40609",
+        0x34 : "INS_ICM42688",
+        0x35 : "INS_ICM42605",
         }
 
     baro_types = {
-        0x01 : "DEVTYPE_BARO_SITL",
-        0x02 : "DEVTYPE_BARO_BMP085",
-        0x03 : "DEVTYPE_BARO_BMP280",
-        0x04 : "DEVTYPE_BARO_BMP388",
-        0x05 : "DEVTYPE_BARO_DPS280",
-        0x06 : "DEVTYPE_BARO_DPS310",
-        0x07 : "DEVTYPE_BARO_FBM320",
-        0x08 : "DEVTYPE_BARO_ICM20789",
-        0x09 : "DEVTYPE_BARO_KELLERLD",
-        0x0A : "DEVTYPE_BARO_LPS2XH",
-        0x0B : "DEVTYPE_BARO_MS5611",
-        0x0C : "DEVTYPE_BARO_SPL06",
-        0x0D : "DEVTYPE_BARO_UAVCAN",
+        0x01 : "BARO_SITL",
+        0x02 : "BARO_BMP085",
+        0x03 : "BARO_BMP280",
+        0x04 : "BARO_BMP388",
+        0x05 : "BARO_DPS280",
+        0x06 : "BARO_DPS310",
+        0x07 : "BARO_FBM320",
+        0x08 : "BARO_ICM20789",
+        0x09 : "BARO_KELLERLD",
+        0x0A : "BARO_LPS2XH",
+        0x0B : "BARO_MS5611",
+        0x0C : "BARO_SPL06",
+        0x0D : "BARO_UAVCAN",
+    }
+
+    airspeed_types = {
+        0x01 : "AIRSPEED_SITL",
+        0x02 : "AIRSPEED_MS4525",
+        0x03 : "AIRSPEED_MS5525",
+        0x04 : "AIRSPEED_DLVR",
+        0x05 : "AIRSPEED_MSP",
+        0x06 : "AIRSPEED_SDP3X",
+        0x07 : "AIRSPEED_UAVCAN",
+        0x08 : "AIRSPEED_ANALOG",
+        0x09 : "AIRSPEED_NMEA",
+        0x0A : "AIRSPEED_ASP5033",
     }
         
     decoded_devname = ""
@@ -435,6 +479,9 @@ def decode_devid(devid, pname):
 
     if pname.startswith("BARO"):
         decoded_devname = baro_types.get(devtype, "UNKNOWN")
+
+    if pname.startswith("ARSP"):
+        decoded_devname = airspeed_types.get(devtype, "UNKNOWN")
         
     print("%s: bus_type:%s(%u)  bus:%u address:%u(0x%x) devtype:%u(0x%x) %s (%u)" % (
         pname,
